-- pNameplates
-- Version:  1.4.0
-- Authors:  Crushbeers, Tourach23

--[[       TABLE OF CONTENTS      ]]--
--[[ ---------------------------- ]]--
--[[ 0. DEFAULT VARIABLE SETTINGS ]]--
--[[ 1. TEXTURES/LOCAL VARIABLES  ]]--
--[[ 2. UTILITY FUNCTIONS		  ]]--
--[[ 3. STYLE FUNCTIONS			  ]]--
--[[ 4. THREAT MANAGEMENT		  ]]--
--[[ 5. UPDATE FUNCTIONS		  ]]--
--[[ 6. CORE FUNCTIONS			  ]]--
--[[ 7. SLASH COMMANDS			  ]]--

--------------------------------------
-- 0. DEFAULT VARIABLE SETTINGS
--------------------------------------

cfg = {
	ShowPercent    = true,
	ShowWhenMax    = true,
	ShowHPValue    = true,
	ThreatEnabled  = true,
	DPSHealerMode  = true,
	FontSize_Perc  = 12,
	FontSize_Name  = 10,
	FontSize_Level = 12,
	FontType       = "Fonts\\ARIALN.ttf",
	
	-- Text Element Positions
	Mvmt = {
		["perc"]  = { "BOTTOM", "TOP"   , 0, 2 , "THINOUTLINE", "CENTER" },
		["name"]  = { "BOTTOM", "BOTTOM", 0, 2 , "NONE",        "CENTER" },
		["level"] = { "LEFT"  , "RIGHT" , 2, 0 , "THINOUTLINE", "LEFT"   }
	},
	
	-- Low Health Warning
	LowHP = {
		[1] = true,     			-- Enabled
		[2] = 25,        			-- Threshold
		[3] = { 1.00, 0.50, 0.00 }, -- Color
		[4] = "orange"				-- Printable Reference
	}
}

--------------------------------------
-- 1. TEXUTURES AND LOCAL VARIABLES
--------------------------------------

local Version = "1.4.0"

local total        = -1
local ForceUpdate  = false
local MoveText     = false
local InCombat     = false
local PVPCombat    = false

local backgroundTexture  	  = "Interface\\Buttons\\WHITE8x8"
local borderDefault      	  = "Interface\\Tooltips\\Nameplate-Border"
local borderTexture      	  = "Interface\\AddOns\\pNameplates\\Textures\\border"
local barTexture         	  = "Interface\\AddOns\\pNameplates\\Textures\\bar"
local React                   = {}
local Players                 = {}

-- Default Unit Colors:
local ClassColors = {
 -- { Red, Green, Blue }
	{ 1.00, 0.00, 0.00 }, -- Hostile
	{ 1.00, 1.00, 0.00 }, -- Neutral
	{ 0.00, 1.00, 0.00 }, -- Friendly NPC
	{ 0.77, 0.12, 0.23 }, -- DK
	{ 1.00, 0.49, 0.04 }, -- Druid
	{ 0.67, 0.83, 0.45 }, -- Hunter
	{ 0.41, 0.80, 0.94 }, -- Mage
	{ 0.33, 0.54, 0.52 }, -- Monk
	{ 0.96, 0.55, 0.73 }, -- Paladin
	{ 1.00, 1.00, 1.00 }, -- Priest
	{ 1.00, 0.96, 0.41 }, -- Rogue
	{ 0.00, 0.44, 0.87 }, -- Shaman
	{ 0.58, 0.51, 0.79 }, -- Warlock
	{ 0.78, 0.61, 0.43 }, -- Warrior
	{ 0.00, 0.00, 1.00 }  -- Friendly Player
}

-- cfg.LowHP Warning Colors
local WarningColors = {
	["purple"] = { 0.60, 0.00, 1.00 },
	["orange"] = { 1.00, 0.50, 0.00 },
	["blue"]   = { 0.00, 0.30, 1.00 }
}

-- Font Variables
local Fonts = {
	["FontType1"] = "ARIALN",
	["FontType2"] = "FRIZQT__",
	["FontType3"] = "MORPHEUS",
	["FontType4"] = "skurri",
}

-- Text Location
local TLoc = {
	["top"]    = { "BOTTOM", "TOP"   ,  0, 2, "THINOUTLINE", "CENTER" },
	["left"]   = { "RIGHT" , "LEFT"  , -2, 0, "THINOUTLINE", "RIGHT"  },
	["right"]  = { "LEFT"  , "RIGHT" ,  2, 0, "THINOUTLINE", "LEFT"   },
	["center"] = { "BOTTOM", "BOTTOM",  0, 2, "NONE",        "CENTER" }
}

-- Threat Setting Lists
local ThreatLow = {
	Right   = { loc = "TOPRIGHT", x = 1, y = 2 },
	Left    = { loc = "BOTTOMLEFT", x = -1, y = -2 },
	Color   = { r = 0.0, g = 0.7, b = 0.0 },
	Texture = "Interface\\AddOns\\pNameplates\\Textures\\threat_LOW",
	HPScale = 1,
}

local ThreatMed = {
	Right   = { loc = "TOPRIGHT", x = 4, y = 5 },
	Left    = { loc = "BOTTOMLEFT", x = -4, y = -5 },
	Color   = { r = 0.7, g = 0.7, b = 0.0 },
	Texture = "Interface\\AddOns\\pNameplates\\Textures\\threat_MED",
	HPScale = 1,
}

local ThreatHigh = {
	Right   = { loc = "TOPRIGHT", x = 4, y = 5 },
	Left    = { loc = "BOTTOMLEFT", x = -4, y = -5 },
	Color   = { r = 0.8, g = 0.0, b = 0.0 },
	Texture = "Interface\\AddOns\\pNameplates\\Textures\\threat_HIGH",
	HPScale = 1.18,
}

--------------------------------------
-- 2. UTILITY FUNCTIONS
--------------------------------------

local function RoundNum(num, dec)
	num = num * math.pow(10, dec)
	num = math.ceil(num)
	num = num / math.pow(10, dec)
	return num
end

local function FormatHP(val)
	if (val >= 1000000) then val = RoundNum((val/1000000), 1).."m"
	elseif (val >= 1000) then val = RoundNum((val/1000), 1).."k" end
	return val
end

local function ParseString(s)
	local loc = strlen(s) - 4
	return string.sub(s, 7, loc)
end

local function AlignObject(object)
	local _, anchor, _, x, y = object:GetPoint()
	object:ClearAllPoints()
	object:SetPoint("BOTTOM", anchor, "BOTTOM", x, y)
end

local function IsNameplate(object)
    local bo = select(2, object:GetRegions())
    if (bo and bo:GetTexture() == borderTexture) then return true end
    if (not bo or bo:GetObjectType() ~= "Texture" or bo:GetTexture() ~= borderDefault) then
      	object.Styled = true
      	return false
    end
    return true
end

--------------------------------------
-- 3. STYLE FUNCTIONS
--------------------------------------

local function StyleTextures(plate)
	if (not plate.Highlight) then return end
	plate.Highlight:SetTexture(1, 1, 1, 0.2)
	plate.Highlight:SetPoint("TOPLEFT", plate.Border, "TOPLEFT", 1, -1)
	plate.Highlight:SetPoint("BOTTOMRIGHT", plate.Border, "BOTTOMRIGHT", -1, 1)
	plate.OldLevel:Hide()
end

local function HideBlizz(plate)
	plate.OldName:Hide()
	plate.Threat:SetTexture(0, 0, 0, 0)
	plate.Dragon:SetTexture(0, 0, 0, 0)
    plate.Castbar.Shield:SetTexture(0, 0, 0, 0)
end

local function StyleCastbar(plate)
	plate.Castbar:SetBackdrop({
        bgFile = backgroundTexture,
        insets = { left = -1, right = -1, top = -1, bottom = -1 }
    })
    plate.Castbar:SetBackdropColor(0, 0, 0, 0.6)
    plate.Castbar:ClearAllPoints()
    plate.Castbar:SetPoint("TOPRIGHT", plate.Health, "BOTTOMRIGHT", 0, -4)
    plate.Castbar:SetPoint("BOTTOMLEFT", plate.Health, "BOTTOMLEFT", 0, -15)
	
	plate.Castbar.Border:SetTexture(0, 0, 0, 1)
    plate.Castbar.Border:SetTexCoord(0, 1, 0, 1)
    plate.Castbar.Border:ClearAllPoints()
    plate.Castbar.Border:SetPoint("TOPRIGHT", plate.Castbar, 0, 0)
    plate.Castbar.Border:SetPoint("BOTTOMLEFT", plate.Castbar, 0, 0)
    plate.Castbar.Border:SetDrawLayer("BORDER")
    
    plate.Castbar.Fill:SetTexture(barTexture)
    plate.Castbar.Fill:ClearAllPoints()
    plate.Castbar.Fill:SetPoint("TOPRIGHT", plate.Castbar.Border, "TOPRIGHT", 1, -1)
    plate.Castbar.Fill:SetPoint("TOPRIGHT", plate.Castbar.Border, "BOTTOMLEFT", -1, 1)
    
    plate.Castbar.Icon:SetSize(12,12)
    plate.Castbar.Icon:ClearAllPoints()
    plate.Castbar.Icon:SetPoint("LEFT", plate.Castbar, "RIGHT", 2, 0)
    
    plate.Castbar.Time = plate.Castbar:CreateFontString(nil, "HIGH")
	plate.Castbar.Time:SetFont(cfg.FontType, 10, "THINOUTLINE")
	plate.Castbar.Time:SetPoint("RIGHT", 0, 0)
	
	plate.Castbar.Spell = plate.Castbar:CreateFontString(nil, "HIGH")
	plate.Castbar.Spell:SetFont(cfg.FontType, 10, "THINOUTLINE")
	plate.Castbar.Spell:SetPoint("LEFT", 1, 0)
end

local function StyleFonts(plate)
	if (cfg.LowHP[1] and plate.Low and InCombat) then return end
	plate.Health.Txt:SetFont(cfg.FontType, cfg.FontSize_Perc, cfg.Mvmt["perc"][5])
	plate.Name:SetFont(cfg.FontType, cfg.FontSize_Name, cfg.Mvmt["name"][5])
	plate.Level:SetFont(cfg.FontType, cfg.FontSize_Level, cfg.Mvmt["level"][5])
end

local function ApplyShadow(text)
	text:SetShadowColor(0, 0, 0, 1)
	text:SetShadowOffset(1, -1)
end

local function RemoveShadow(text)
	text:SetShadowColor(0, 0, 0, 0)
end

local function StyleLevelText(plate)
	plate.Level:SetTextColor(plate.OldLevel:GetTextColor())
	if (plate.Boss:IsShown()) then plate.Level:Hide()
	else plate.Level:Show() end
	
	local string = plate.OldLevel:GetText()
	if (not string) then string = "" end
	return string
end

--------------------------------------
-- 4. THREAT MANAGEMENT
--------------------------------------

local function AssignDefaultColor(plate)
	local r = RoundNum(select(1, plate.Health:GetStatusBarColor()), 2)
	local g = RoundNum(select(2, plate.Health:GetStatusBarColor()), 2)
	local b = RoundNum(select(3, plate.Health:GetStatusBarColor()), 2)
	
	Players[plate.Text] = false -- Initialize to false
	
	for i,v in ipairs(ClassColors) do
		if (r == v[1] and g == v[2] and b == v[3]) then
			React[plate.Text] = v
			if (i > 3) then Players[plate.Text] = true end
			break
		end
	end
end

local function SetThreatStyle(plate, Threat)
	plate.Health:SetStatusBarColor(Threat.Color.r, Threat.Color.g, Threat.Color.b, 1)
	plate.ThreatBorder:SetPoint(Threat.Right.loc, plate.Border, Threat.Right.x, Threat.Right.y)
	plate.ThreatBorder:SetPoint(Threat.Left.loc, plate.Border, Threat.Left.x, Threat.Left.y)
	plate.ThreatBorder:SetTexture(Threat.Texture)
	plate.Health:SetScale(Threat.HPScale)
end

local function UpdateThreat(plate)
	if (Players[plate.Text] == nil) then AssignDefaultColor(plate) end
	if (not InCombat and plate.ThreatBorder:IsShown()) or (Players[plate.Text] or not cfg.ThreatEnabled) then
		plate.ThreatBorder:Hide()
		plate.Health:SetStatusBarColor(React[plate.Text][1], React[plate.Text][2], React[plate.Text][3], 1)
		plate.Health:SetScale(1)
		return 
	elseif (InCombat) then
		plate.ThreatBorder:Show()
	else return end -- Returns if out of combat AND threat objects are hidden
	
	local red, green, blue, _ = plate.Threat:GetVertexColor()
	local shown = plate.Threat:IsShown()
	
	if (not shown) then 
		if (cfg.DPSHealerMode) then SetThreatStyle(plate, ThreatLow)
		else SetThreatStyle(plate, ThreatHigh) end
	elseif (red > 0) then
		if (green > 0) then SetThreatStyle(plate, ThreatMed) return end
		if (cfg.DPSHealerMode) then SetThreatStyle(plate, ThreatHigh)
		else SetThreatStyle(plate, ThreatLow) end
	end
end

--------------------------------------
-- 5. UPDATE FUNCTIONS
--------------------------------------

local function UpdateDisplay(plate)
	plate.Text = plate.OldName:GetText()
	
	plate.Health:SetHeight(14)
	plate.Name:SetText(plate.Text)
	plate.Level:SetText(StyleLevelText(plate))
	
	local _, max = plate.Health:GetMinMaxValues()
	local value = plate.Health:GetValue()
	local perc = math.floor((value/max)*100)
	plate.Low = (cfg.LowHP[1] and perc <= cfg.LowHP[2])
	
	if (cfg.ThreatEnabled or ForceUpdate) then UpdateThreat(plate) end
	if (InCombat and plate.Low) then
		plate.Health.Txt:SetFont(cfg.FontType, cfg.FontSize_Perc+3, cfg.Mvmt["perc"][5])
		plate.Health.Txt:SetTextColor(cfg.LowHP[3][1], cfg.LowHP[3][2], cfg.LowHP[3][3])
	else
		plate.Health.Txt:SetFont(cfg.FontType, cfg.FontSize_Perc, cfg.Mvmt["perc"][5])
		plate.Health.Txt:SetTextColor(1, 1, 1)
	end
	
	if (cfg.ShowPercent) then
		if (perc >= 100 and not cfg.ShowWhenMax) then plate.Health.Txt:SetText("")
		else
			if (cfg.ShowHPValue) then plate.Health.Txt:SetText(FormatHP(value).." / "..perc.."%")
			else plate.Health.Txt:SetText(perc.."%") end
		end
	else
		plate.Health.Txt:SetText("")
	end
end

local function UpdateCastInfo(Castbar)
	local SpellName = select(3, UnitCastingInfo("target"))
	if (not SpellName) then SpellName = select(3, UnitChannelInfo("target")) end
	local _, max = Castbar:GetMinMaxValues()
	local current = Castbar:GetValue()
	local final = math.floor((max-current)*10)/10
	
	Castbar.Time:SetText(final)
	Castbar.Spell:SetText(SpellName)
	
	-- Fixes phantom castbar frames appearing
	if (not final or final <= 0) then Castbar:Hide()
	else Castbar:Show() end
end

local function UpdateTextLocations(plate)
	plate.Health.Txt:ClearAllPoints()
	plate.Health.Txt:SetPoint(cfg.Mvmt["perc"][1], plate.Health, cfg.Mvmt["perc"][2], cfg.Mvmt["perc"][3], cfg.Mvmt["perc"][4])
	plate.Health.Txt:SetJustifyH(cfg.Mvmt["perc"][6])
	if (cfg.Mvmt["perc"][5] == "NONE") then ApplyShadow(plate.Health.Txt)
	else RemoveShadow(plate.Health.Txt) end
	
	plate.Name:ClearAllPoints()
	plate.Name:SetPoint(cfg.Mvmt["name"][1], plate.Health, cfg.Mvmt["name"][2], cfg.Mvmt["name"][3], cfg.Mvmt["name"][4])
	plate.Name:SetJustifyH(cfg.Mvmt["name"][6])
	if (cfg.Mvmt["name"][5] == "NONE") then ApplyShadow(plate.Name)
	else RemoveShadow(plate.Name) end
	
	plate.Level:ClearAllPoints()
	plate.Level:SetPoint(cfg.Mvmt["level"][1], plate.Health, cfg.Mvmt["level"][2], cfg.Mvmt["level"][3], cfg.Mvmt["level"][4])
	plate.Level:SetJustifyH(cfg.Mvmt["level"][6])
	if (cfg.Mvmt["level"][5] == "NONE") then ApplyShadow(plate.Level)
	else RemoveShadow(plate.Level) end
	
	plate.Boss:ClearAllPoints()
	plate.Boss:SetPoint(plate.Level:GetPoint())
end

--------------------------------------
-- 6. CORE FUNCTIONS
--------------------------------------

local function init(plate)
	plate:SetToplevel(true)

	plate.Health, plate.Castbar = plate:GetChildren()
    plate.Castbar.Fill, plate.Castbar.Border, plate.Castbar.Shield, plate.Castbar.Icon = plate.Castbar:GetRegions()
    plate.Threat, plate.Border, plate.Highlight, plate.OldName, plate.OldLevel, plate.Boss, _, plate.Dragon = plate:GetRegions()
	
	plate.Border:SetTexture(borderTexture)
    plate.Border:SetVertexColor(0, 0, 0)
    plate.Border:ClearAllPoints()
    plate.Border:SetPoint("TOPRIGHT", plate.Health, 1, 1)
    plate.Border:SetPoint("BOTTOMLEFT", plate.Health, -1, -1)
    
    plate.ThreatBorder = plate:CreateTexture(nil, "Texture")
    plate.ThreatBorder:SetPoint("TOPRIGHT", plate.Border, 1, 2)
    plate.ThreatBorder:SetPoint("BOTTOMLEFT", plate.Border, -1, -2)
    
	plate.Health:SetHeight(14)
	plate.Health:SetStatusBarTexture(barTexture)
    plate.Health:SetBackdrop({
        bgFile = backgroundTexture,
        insets = { left = 0, right = 0, top = 0, bottom = 0 }
    })
    plate.Health:SetBackdropColor(0, 0, 0, 0.6)
	
	plate.Level = plate:CreateFontString(nil, "HIGH")
	plate.Level:SetFont(cfg.FontType, cfg.FontSize_Level, cfg.Mvmt["level"][5])
	
	plate.Name = plate:CreateFontString(nil, "HIGH")
	plate.Name:SetFont(cfg.FontType, cfg.FontSize_Name, cfg.Mvmt["name"][5])
	plate.Name:SetText(plate.OldName:GetText())
	plate.Name:SetWordWrap(false)
	plate.Name:SetWidth(100)
	
	plate.Health.Txt = plate.Health:CreateFontString(nil, "HIGH")
	plate.Health.Txt:SetFont(cfg.FontType, cfg.FontSize_Perc, cfg.Mvmt["perc"][5])
	
	-- Initialization Stuff
	plate.Mod = true
	UpdateTextLocations(plate)
	UpdateDisplay(plate)
	StyleTextures(plate)
	AlignObject(plate.Health)
	StyleCastbar(plate)
	HideBlizz(plate)
	
	plate.Castbar:SetScript("OnValueChanged", UpdateCastInfo)
	
	plate.Health:SetScript("OnShow", function()
		UpdateDisplay(plate)
		StyleTextures(plate)
		UpdateCastInfo(plate.Castbar)
		AlignObject(plate.Health)
	end)
	plate.Health:SetScript("OnHide", function() UpdateDisplay(plate) end)
	plate.Health:SetScript("OnValueChanged", function() UpdateDisplay(plate) end)
	plate.Health:SetScript("OnMinMaxChanged", function() UpdateDisplay(plate) end)
end

local function GetObjects(plate, elapsed)
    local current = select("#", WorldFrame:GetChildren())
    if (current ~= total or ForceUpdate or MoveText) then
    	total = current
	  	for i = 1, current do
			local plate = select(i, WorldFrame:GetChildren())
			if (IsNameplate(plate)) then
		  		if (not plate.Mod) then init(plate)
		  		else 
		  			UpdateDisplay(plate)
		  			StyleTextures(plate)
		  			if (ForceUpdate or MoveText) then StyleFonts(plate) end
		  			if (MoveText) then UpdateTextLocations(plate) end
		  		end
		  	end
		end
		ForceUpdate = false
		MoveText = false
    end
end

local f = CreateFrame("Frame")
f:SetScript("OnEvent", function(plate, event)
	ForceUpdate = true
	if (event == "PLAYER_LOGIN") then
		SetCVar("ShowClassColorInNameplate", 1)
		plate:SetScript("OnUpdate", GetObjects)
	elseif (event == "PLAYER_REGEN_DISABLED") then
		InCombat = true
	elseif (event == "PLAYER_REGEN_ENABLED") then
		InCombat = false
	end
	collectgarbage()
end)
  
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("PLAYER_REGEN_DISABLED") -- Enter Combat
f:RegisterEvent("PLAYER_REGEN_ENABLED")  -- Leave Combat

--------------------------------------
-- 7. SLASH COMMANDS
--------------------------------------

local function handler(args)
	local cur = nil
	local cmd = string.split(' ', args):lower()
	if (not cmd or cmd == "") then
		print("|cFF48D1CCp|rNameplates |cFF48D1CCCommand List:|r")
		print("|cFF48D1CC/pp percent|r, to toggle Health Percents.")
		print("|cFF48D1CC/pp value|r, to toggle Health Value. |cFF48D1CC(dependant on Percent)|r")
		print("|cFF48D1CC/pp max|r, to toggle 100% Health. |cFF48D1CC(dependant on Percent)|r")
		print("|cFF48D1CC/pp low|r, to config Low Health Warning.")
		print("|cFF48D1CC/pp font|r, to config Fonts.")
		print("|cFF48D1CC/pp move|r, to config Text Locations.")
		print("|cFF48D1CC/pp role|r, to config Threat Role.")
		print("|cFF48D1CC/pp threat|r, to toggle Threat Coloring.")
		print("|cFF48D1CC/pp motion|r, to config Nameplate Motion.")
		--print("|cFF48D1CC/pp enemy|r, to config Enemy Nameplate Visibility.")
	elseif (cmd:match"font") then
		cmd = select(2, string.split(' ', args))
		if (not cmd) then 
			cur = ParseString(cfg.FontType)
			print("|cFF48D1CCp|rNameplates |cFF48D1CCFont commands:  (Current Setting:|r "..cur.."|cFF48D1CC)|r")
			print("|cFF48D1CC/pp font size|r, to see Font Size options.")
			print("|cFF48D1CC/pp font type|r, to see Font Type options.")
		elseif (cmd:match"size") then
			local NewSize = nil
			cmd = select(3, string.split(' ', args))
			if (not cmd) then
				print("|cFF48D1CCp|rNameplates |cFF48D1CCFont Size commands:|r")
				print("|cFF48D1CC/pp font size perc ##|r, to set Percent Font Size.")
				print("|cFF48D1CC/pp font size name ##|r, to set Unit Name Font Size.")
				print("|cFF48D1CC/pp font size level ##|r, to set Unit Level Font Size.")
				print("|cFFCB4040Note|r: If you enter a non-number, you will get an LUA error.")
			else 
				NewSize = (select(4, string.split(' ', args)) + 0)
				if (cmd:match"perc") then 
					cfg.FontSize_Perc = NewSize
					print("|cFF48D1CCp|rNameplates |cFF48D1CCPercent Font size set to|r "..cfg.FontSize_Perc.."|cFF48D1CC.|r")
				elseif (cmd:match"name") then
					cfg.FontSize_Name = NewSize
					print("|cFF48D1CCp|rNameplates |cFF48D1CCName Font size set to|r "..cfg.FontSize_Name.."|cFF48D1CC.|r")
				elseif (cmd:match"level") then
					cfg.FontSize_Level = NewSize
					print("|cFF48D1CCp|rNameplates |cFF48D1CCLevel Font size set to|r "..cfg.FontSize_Level.."|cFF48D1CC.|r")
				else
					print("|cFFCB4040Invalid Entry|r: |cFF48D1CC/pp font size|r, to see Font Size options again.")
					print("|cFFCB4040Text Entered|r: /pp "..args)
				end
			end
		elseif (cmd:match"type") then
			cmd = select(3, string.split(' ', args))
			if (not cmd) then
				print("|cFF48D1CCp|rNameplates |cFF48D1CCFont Type commands:|r ")
				print("|cFF48D1CC/pp font type #|r, (1-4) to select Font Type.")
				print("|cFF48D1CCFont Types|r:")
				print("|cFF48D1CC1|r. "..Fonts["FontType1"])
				print("|cFF48D1CC2|r. "..Fonts["FontType2"])
				print("|cFF48D1CC3|r. "..Fonts["FontType3"])
				print("|cFF48D1CC4|r. "..Fonts["FontType4"])
			elseif (not cmd:match"1" and not cmd:match"2" and not cmd:match"3" and not cmd:match"4") then
				print("|cFFCB4040Invalid Entry|r: |cFF48D1CC/pp font type|r, to see Font Type options again.")
				print("|cFFCB4040Text Entered|r: /pp "..args)
			else
				FontType = "Fonts\\"..Fonts["FontType"..cmd]..".ttf"
				print("|cFF48D1CCp|rNameplates |cFF48D1CCFont type changed to |r"..Fonts["FontType"..cmd].."|cFF48D1CC.|r")
			end
		else
			print("|cFFCB4040Invalid Entry|r: |cFF48D1CC/pp font|r, to see Font commands again.")
			print("|cFFCB4040Text Entered|r: /pp "..args)
		end
	elseif (cmd:match"move") then
		cmd = select(2, string.split(' ', args))
		if (not cmd) then
			print("|cFF48D1CCp|rNameplates |cFF48D1CCMovement commands:|r ")
			print("|cFF48D1CCValid Locations|r: top, left, right, center")
    		print("|cFF48D1CC/pp move perc 'location'|r, to relocate Percent Text.")
    		print("|cFF48D1CC/pp move name 'location'|r, to relocate Name Text.")
    		print("|cFF48D1CC/pp move level 'location'|r, to relocate Level Text.")
    	else
    		cur = cmd
    		cmd = select(3, string.split(' ', args))
			if (not cmd) then cmd = "error" end
			if (cmd:match"top" or cmd:match"left" or cmd:match"right" or cmd:match"center") then
				cfg.Mvmt[cur] = TLoc[cmd]
				MoveText = true
			else 
				print("|cFFCB4040Invalid Entry|r: |cFF48D1CC/pp move|r, to see Movement options again.")
				print("|cFFCB4040Text Entered|r: /pp "..args)
			end
    	end
    elseif (cmd:match"low") then
		cmd = select(2, string.split(' ', args))
		if (not cmd) then
			print("|cFF48D1CCp|rNameplates |cFF48D1CCLow HP Warning commands:|r ")
    		print("|cFF48D1CC/pp low on|r, to enable Low HP Warning.")
    		print("|cFF48D1CC/pp low off|r, to disable Low HP Warning.")
    		print("|cFF48D1CC/pp low limit ##|r, to set Warning Threshold Percent.  |cFF48D1CC(Currently:|r "..cfg.LowHP[2].."|cFF48D1CC)|r")
    		print("|cFF48D1CC/pp low color|r, to see Warning Color options.  |cFF48D1CC(Currently:|r "..cfg.LowHP[4].."|cFF48D1CC)|r")
    		print("|cFFCB4040Note|r: If you enter a non-number for the limit, you will get an LUA error.")
    	elseif (cmd:match"on") then
    		cfg.LowHP[1] = true
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: Low HP Warning|r enabled|cFF48D1CC.|r")
    	elseif (cmd:match"off") then
    		cfg.LowHP[1] = false
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: Low HP Warning|r disabled|cFF48D1CC.|r")
    	elseif (cmd:match"limit") then
    		cmd = select(3, string.split(' ', args))
    		if (not cmd) then 
    			print("|cFFCB4040Invalid Entry|r: You did not enter a number for the limit.")
    			print("|cFFCB4040Text Entered|r: /pp "..args)
    		else
    			cfg.LowHP[2] = cmd + 0
    			print("|cFF48D1CCp|rNameplates|cFF48D1CC: Low HP Threashold set to |r"..cfg.LowHP[2].."|cFF48D1CC.|r")
    		end
    	elseif (cmd:match"color") then
    		cmd = select(3, string.split(' ', args))
			if (not cmd) then
				print("|cFF48D1CCp|rNameplates |cFF48D1CCLow HP Color commands:|r ")
				print("|cFF48D1CC/pp low color 'color'|r, to set Low HP Warning Color.")
				print("|cFF48D1CCCurrent Color|r: "..cfg.LowHP[4])
				print("|cFF48D1CCValid Colors|r: |cFF9900FFpurple|r, |cFFFF7F00orange|r, |cFF004DFFblue|r")
			elseif (cmd:match"purple" or cmd:match"orange" or cmd:match"blue") then
				cfg.LowHP[3] = WarningColors[cmd]
				cfg.LowHP[4] = cmd
				print("|cFF48D1CCp|rNameplates|cFF48D1CC: Low HP Color set to |r"..cmd.."|cFF48D1CC.|r")
			else
				print("|cFFCB4040Invalid Entry|r: |cFF48D1CC/pp low color|r, to see Low HP Color options again.")
				print("|cFFCB4040Text Entered|r: /pp "..args)
			end
		else 
			print("|cFFCB4040Invalid Entry|r: |cFF48D1CC/pp low|r, to see Low HP Warning options again.")
    		print("|cFFCB4040Text Entered|r: /pp "..args)
    	end
    elseif (cmd:match"percent") then
    	if (cfg.ShowPercent) then
			cfg.ShowPercent = false
			print("|cFF48D1CCp|rNameplates|cFF48D1CC: Health percent|r disabled|cFF48D1CC.|r")
		else
			cfg.ShowPercent = true
			print("|cFF48D1CCp|rNameplates|cFF48D1CC: Health percent|r enabled|cFF48D1CC.|r")
		end
	elseif (cmd:match"value") then
    	if (cfg.ShowHPValue) then
			cfg.ShowHPValue = false
			print("|cFF48D1CCp|rNameplates|cFF48D1CC: Health value|r disabled|cFF48D1CC.|r")
		else
			cfg.ShowHPValue = true
			print("|cFF48D1CCp|rNameplates|cFF48D1CC: Health value|r enabled|cFF48D1CC.|r")
		end
    elseif (cmd:match"max") then
    	if (cfg.ShowWhenMax) then 
    		cfg.ShowWhenMax = false
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: 100% Health display|r disabled|cFF48D1CC|r")
    	else
    		cfg.ShowWhenMax = true
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: 100% Health display|r enabled|cFF48D1CC.|r")
    	end
    elseif (cmd:match"threat") then
    	if (cfg.ThreatEnabled) then 
    		cfg.ThreatEnabled = false
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: Threat coloring|r disabled|cFF48D1CC.|r")
    	else
    		cfg.ThreatEnabled = true
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: Threat coloring|r enabled|cFF48D1CC.|r")
    	end
    elseif (cmd:match"role") then
    	cmd = select(2, string.split(' ', args))
    	if (not cmd) then
    		if (cfg.DPSHealerMode) then cur = "DPS/Heal mode"
    		else cur = "Tank mode" end
    		print("|cFF48D1CCp|rNameplates |cFF48D1CCRole commands:|r  (Current Setting: "..cur..")")
    		print("|cFF48D1CC/pp role tank|r, for Tank mode.")
    		print("|cFF48D1CC/pp role dps OR heal|r, for DPS/Healing mode.")
    	elseif (cmd:match"tank") then
    		cfg.DPSHealerMode = false
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: Tank mode|r enabled|cFF48D1CC.|r")
    	elseif (cmd:match"dps" or cmd:match"heal") then
    		cfg.DPSHealerMode = true
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: DPS/Healing mode|r enabled|cFF48D1CC.|r")
    	else
    		print("|cFFCB4040Invalid Entry|r: |cFF48D1CC/pp role|r, to see Role options again.")
    		print("|cFFCB4040Text Entered|r: /pp "..args)
    	end
    elseif (cmd:match"motion") then
    	cmd = select(2, string.split(' ', args))
    	if (not cmd) then
    		cur = GetCVar("nameplateMotion")
    		print("|cFF48D1CCp|rNameplates |cFF48D1CCMotion commands:|r  (Current Setting: "..cur..")")
    		print("|cFF48D1CC/pp motion 0|r, for Overlap.")
    		print("|cFF48D1CC/pp motion 1|r, for Stacking.")
    		print("|cFF48D1CC/pp motion 2|r, for Spreading.")
    	elseif (cmd:match"0") then
    		SetCVar("nameplateMotion", 0)
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: Nameplate Motion set to |r Overlap|cFF48D1CC.|r")
    	elseif (cmd:match"1") then
    		SetCVar("nameplateMotion", 1)
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: Nameplate Motion set to |r Stacking|cFF48D1CC.|r")
    	elseif (cmd:match"2") then
    		SetCVar("nameplateMotion", 2)
    		print("|cFF48D1CCp|rNameplates|cFF48D1CC: Nameplate Motion set to |r Spreading|cFF48D1CC.|r")
    	else
    		print("|cFFCB4040Invalid Entry|r: |cFF48D1CC/pp motion|r, to see Motion options again.")
    		print("|cFFCB4040Text Entered|r: /pp "..args)
    	end
    else
		print("|cFFCB4040Invalid Entry|r: |cFF48D1CC/pp|r, to see Command options again.")
		print("|cFFCB4040Text Entered|r: /pp "..args)
    end
    ForceUpdate = true
end
 
SLASH_pp1 = "/pp";
SlashCmdList["pp"] = handler;

print("|cFF48D1CCp|rNameplates  |cFF48D1CC(v"..Version..")|r  is loaded: |cFF48D1CC/pp|r for help.")